package MT::Plugin::MajongUtil;

use strict;
use MT;
use MT::Plugin;
use MT::Template::Context;

use vars qw($VERSION);
$VERSION = '0.1';

@MT::Plugin::MajongUtil::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::MajongUtil({
  name => 'MajongUtil',
  version => $VERSION,
  description => 'translate majong tile markup to HTML img tags.',
  doc_link => 'http://jamong.s285.xrea.com/archives/2007/06/majongutil_01.html',
});

MT->add_plugin($plugin);

MT::Template::Context->add_global_filter(mj_trans => \&mj_trans);

sub mj_trans {
    my ($text, $arg, $ctx) = @_;
    if (!$arg) { return $text; }

    my $app = new MT::App::CMS;
    my $img_path = $app->static_path . 'plugins/MajongUtil/tiles/';

    $text =~ s/\|([1-9bcdtnspwgrx\-\[\] ]+)\|/MT::Plugin::MajongUtil::str2tileimg($1,$img_path)/ige;
    return $text;
}

sub str2tileimg {
    my ($str,$img_path) = @_;
    my $original = $str;
    my %_honor_chr = ( t => 27,
                       n => 28,
                       s => 29,
                       p => 30,
                       w => 31,
                       g => 32,
                       r => 33,
                       x => 34,);

    my %_suit_base = ( c => -1,
                       d => 8,
                       b => 17, );

    my @_tile_img = (
        '1m', '2m', '3m', '4m', '5m', '6m', '7m', '8m', '9m', 
        '1p', '2p', '3p', '4p', '5p', '6p', '7p', '8p', '9p', 
        '1s', '2s', '3s', '4s', '5s', '6s', '7s', '8s', '9s',
        't',  'n',  's',  'p',  'w',  'g',  'r',  'x',
    );

    chomp($str);
    $str =~ s/ //g;
    my @tiles = ();
    my $suit_mode = 0;

    my @open_tiles = ();
    while( $str =~ s/\[(.*?)\]// ){
        my $open_tile = str2tileimg($1);
        push @open_tiles, $open_tile;
    }
    my $laid = 0;
    while($str){
        my $chr = lc chop $str;
        my $attr = 0;
        if ($laid) {
            $attr += 0x0100;
            $laid = 0;
        }
        if ($chr eq '-'){
            $laid = 1;
        }
        elsif(defined(my $suit = $_suit_base{$chr})){
            $suit_mode = $suit;
        }
        elsif(defined(my $honor = $_honor_chr{$chr})){
            unshift @tiles, $honor + $attr;
        }
        elsif($chr =~ /[1-9]/){
            return $original unless $suit_mode;
            unshift @tiles, $suit_mode + $chr + $attr;
        }
        else {
            return $original;
        }
    }
    
    my $result;
    foreach (@tiles) {
        my $prefix = '';
        if ($_ & 0x0100) {
            $_ -= 0x0100;
            $prefix = 'y';
        } 
        $result .= '<img src="' . $img_path . $prefix . $_tile_img[$_] . '.gif" />';
    }

    foreach (@open_tiles) {
        $result .= ' <span class="open_tiles">' . $_ . '</span>';
    }
    return $result;
}

1;
